@@ -6,15 +6,14 @@
     Allow graphviz-formatted graphs to be included in Sphinx-generated
     documents inline.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import os
 import re
-import sys
 import posixpath
 from os import path
+from math import ceil
 from subprocess import Popen, PIPE
 try:
     from hashlib import sha1 as sha
@@ -22,13 +21,15 @@ except ImportError:
     from sha import sha
 
 from docutils import nodes
+from docutils.parsers.rst import directives
 
 from sphinx.errors import SphinxError
-from sphinx.util import ensuredir
+from sphinx.util.osutil import ensuredir, ENOENT, EPIPE
 from sphinx.util.compat import Directive
 
 
 mapname_re = re.compile(r'<map id="(.*?)"')
+svg_dim_re = re.compile(r'<svg\swidth="(\d+)pt"\sheight="(\d+)pt"', re.M)
 
 
 class GraphvizError(SphinxError):
@@ -47,12 +48,21 @@ class Graphviz(Directive):
     required_arguments = 0
     optional_arguments = 0
     final_argument_whitespace = False
-    option_spec = {}
+    option_spec = {
+        'alt': directives.unchanged,
+    }
 
     def run(self):
+        dotcode = '\n'.join(self.content)
+        if not dotcode.strip():
+            return [self.state_machine.reporter.warning(
+                'Ignoring "graphviz" directive without content.',
+                line=self.lineno)]
         node = graphviz()
-        node['code'] = '\n'.join(self.content)
+        node['code'] = dotcode
         node['options'] = []
+        if 'alt' in self.options:
+            node['alt'] = self.options['alt']
         return [node]
 
 
@@ -64,13 +74,17 @@ class GraphvizSimple(Directive):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = False
-    option_spec = {}
+    option_spec = {
+        'alt': directives.unchanged,
+    }
 
     def run(self):
         node = graphviz()
         node['code'] = '%s %s {\n%s\n}\n' % \
                        (self.name, self.arguments[0], '\n'.join(self.content))
         node['options'] = []
+        if 'alt' in self.options:
+            node['alt'] = self.options['alt']
         return [node]
 
 
@@ -95,10 +109,14 @@ def render_dot(self, code, options, format, prefix='graphviz'):
 
     if hasattr(self.builder, '_graphviz_warned_dot') or \
        hasattr(self.builder, '_graphviz_warned_ps2pdf'):
-        return None
+        return None, None
 
     ensuredir(path.dirname(outfn))
 
+    # graphviz expects UTF-8 by default
+    if isinstance(code, unicode):
+        code = code.encode('utf-8')
+
     dot_args = [self.builder.config.graphviz_dot]
     dot_args.extend(self.builder.config.graphviz_dot_args)
     dot_args.extend(options)
@@ -108,26 +126,69 @@ def render_dot(self, code, options, format, prefix='graphviz'):
     try:
         p = Popen(dot_args, stdout=PIPE, stdin=PIPE, stderr=PIPE)
     except OSError, err:
-        if err.errno != 2:   # No such file or directory
+        if err.errno != ENOENT:   # No such file or directory
             raise
         self.builder.warn('dot command %r cannot be run (needed for graphviz '
                           'output), check the graphviz_dot setting' %
                           self.builder.config.graphviz_dot)
         self.builder._graphviz_warned_dot = True
-        return None
-    # graphviz expects UTF-8 by default
-    if isinstance(code, unicode):
-        code = code.encode('utf-8')
-    stdout, stderr = p.communicate(code)
+        return None, None
+    try:
+        # Graphviz may close standard input when an error occurs,
+        # resulting in a broken pipe on communicate()
+        stdout, stderr = p.communicate(code)
+    except OSError, err:
+        if err.errno != EPIPE:
+            raise
+        # in this case, read the standard output and standard error streams
+        # directly, to get the error message(s)
+        stdout, stderr = p.stdout.read(), p.stderr.read()
+        p.wait()
     if p.returncode != 0:
         raise GraphvizError('dot exited with error:\n[stderr]\n%s\n'
                             '[stdout]\n%s' % (stderr, stdout))
     return relfn, outfn
 
 
-def render_dot_html(self, node, code, options, prefix='graphviz', imgcls=None):
+def get_svg_tag(svgref, svgfile, imgcls=None):
+    # Webkit can't figure out svg dimensions when using object tag
+    # so we need to get it from the svg file
+    fp = open(svgfile, 'r')
     try:
-        fname, outfn = render_dot(self, code, options, 'png', prefix)
+        for line in fp:
+            match = svg_dim_re.match(line)
+            if match:
+                dimensions = match.groups()
+                break
+        else:
+            dimensions = None
+    finally:
+        fp.close()
+
+    # We need this hack to make WebKit show our object tag properly
+    def pt2px(x):
+        return int(ceil((96.0/72.0) * float(x)))
+
+    if dimensions:
+        style = ' width="%s" height="%s"' % tuple(map(pt2px, dimensions))
+    else:
+        style = ''
+
+    # The object tag works fine on Firefox and WebKit
+    # Besides it's a hack, this strategy does not mess with templates.
+    imgcss = imgcls and ' class="%s"' % imgcls or ''
+    return '<object type="image/svg+xml" data="%s"%s%s/>\n' % \
+           (svgref, imgcss, style)
+
+
+def render_dot_html(self, node, code, options, prefix='graphviz',
+                    imgcls=None, alt=None):
+    format = self.builder.config.graphviz_output_format
+    try:
+        if format not in ('png', 'svg'):
+            raise GraphvizError("graphviz_output_format must be one of 'png', "
+                                "'svg', but is %r" % format)
+        fname, outfn = render_dot(self, code, options, format, prefix)
     except GraphvizError, exc:
         self.builder.warn('dot code %r: ' % code + str(exc))
         raise nodes.SkipNode
@@ -136,23 +197,29 @@ def render_dot_html(self, node, code, options, prefix='graphviz', imgcls=None):
     if fname is None:
         self.body.append(self.encode(code))
     else:
-        mapfile = open(outfn + '.map', 'rb')
-        try:
-            imgmap = mapfile.readlines()
-        finally:
-            mapfile.close()
-        imgcss = imgcls and 'class="%s"' % imgcls or ''
-        if len(imgmap) == 2:
-            # nothing in image map (the lines are <map> and </map>)
-            self.body.append('<img src="%s" alt="%s" %s/>\n' %
-                             (fname, self.encode(code).strip(), imgcss))
+        if alt is None:
+            alt = node.get('alt', self.encode(code).strip())
+        if format == 'svg':
+            svgtag = get_svg_tag(fname, outfn, imgcls)
+            self.body.append(svgtag)
         else:
-            # has a map: get the name of the map and connect the parts
-            mapname = mapname_re.match(imgmap[0]).group(1)
-            self.body.append('<img src="%s" alt="%s" usemap="#%s" %s/>\n' %
-                             (fname, self.encode(code).strip(),
-                              mapname, imgcss))
-            self.body.extend(imgmap)
+            mapfile = open(outfn + '.map', 'rb')
+            try:
+                imgmap = mapfile.readlines()
+            finally:
+                mapfile.close()
+            imgcss = imgcls and 'class="%s"' % imgcls or ''
+            if len(imgmap) == 2:
+                # nothing in image map (the lines are <map> and </map>)
+                self.body.append('<img src="%s" alt="%s" %s/>\n' %
+                                 (fname, alt, imgcss))
+            else:
+                # has a map: get the name of the map and connect the parts
+                mapname = mapname_re.match(imgmap[0]).group(1)
+                self.body.append('<img src="%s" alt="%s" usemap="#%s" %s/>\n' %
+                                 (fname, alt, mapname, imgcss))
+                self.body.extend(imgmap)
+
     self.body.append('</p>\n')
     raise nodes.SkipNode
 
@@ -185,3 +252,4 @@ def setup(app):
     app.add_directive('digraph', GraphvizSimple)
     app.add_config_value('graphviz_dot', 'dot', 'html')
     app.add_config_value('graphviz_dot_args', [], 'html')
+    app.add_config_value('graphviz_output_format', 'png', 'html')
