@@ -35,7 +35,7 @@ ENV_WARNINGS = """\
 %(root)s/images.txt:23: WARNING: nonlocal image URI found: \
 http://www.python.org/logo.png
 %(root)s/includes.txt:\\d*: \\(WARNING/2\\) Encoding 'utf-8-sig' used for \
-reading included file u'wrongenc.inc' seems to be wrong, try giving an \
+reading included file u'.*?wrongenc.inc' seems to be wrong, try giving an \
 :encoding: option\\n?
 %(root)s/includes.txt:4: WARNING: download file not readable: nonexisting.png
 %(root)s/objects.txt:\\d*: WARNING: using old C markup; please migrate to \
